@@ -25,7 +25,7 @@
   * @author Samuel Mimram
   *)
 
-(* $Id: vorbis.ml 5149 2008-02-25 09:01:12Z smimram $ *)
+(* $Id: vorbis.ml 6403 2009-02-17 18:23:30Z metamorph68 $ *)
 
 type encoder
 
@@ -46,6 +46,8 @@ exception Internal_fault
 exception Invalid_argument
 exception Not_implemented
 exception Unknown_error of int
+exception Not_audio
+exception False
 exception Utf8_failure of string
 
 let _ =
@@ -65,7 +67,9 @@ let _ =
   Callback.register_exception "vorbis_exn_internal_fault" Internal_fault;
   Callback.register_exception "vorbis_exn_invalid" Invalid_argument;
   Callback.register_exception "vorbis_exn_not_implemented" Not_implemented;
+  Callback.register_exception "vorbis_exn_not_audio" Not_audio;
   Callback.register_exception "vorbis_exn_unknown_error" (Unknown_error 0);
+  Callback.register_exception "vorbis_exn_false" False;
   Callback.register_exception "vorbis_exn_utf8_failure" (Utf8_failure "")
 
 let tags ?title ?artist ?genre ?date ?album ?tracknumber ?comment () =
@@ -96,14 +100,23 @@ struct
 
   external reset : t -> unit = "ocaml_vorbis_reset"
 
-  external headerout : t -> Ogg.Stream.t -> (string * string) array -> unit = "ocaml_vorbis_analysis_headerout"
+
+  external headerout_packetout : t -> (string * string) array -> Ogg.Stream.packet*Ogg.Stream.packet*Ogg.Stream.packet =  "ocaml_vorbis_analysis_headerout"
+
+  let headerout_packetout state tags =
+    let tags = Array.of_list (tags@[("ENCODER", encoder_tag)]) in
+      headerout_packetout state tags
 
   let headerout state os tags =
-    let tags = Array.of_list (tags@[("ENCODER", encoder_tag)]) in
-      headerout state os tags
+    let p1,p2,p3 = headerout_packetout state tags in
+    Ogg.Stream.put_packet os p1;
+    Ogg.Stream.put_packet os p2;
+    Ogg.Stream.put_packet os p3 
 
   external encode_buffer_float : t -> Ogg.Stream.t -> float array array -> int -> int -> unit = "ocaml_vorbis_encode_float"
 
+  external time_of_granulepos : t -> Int64.t -> Nativeint.t = "ocaml_vorbis_encode_time_of_granulepos"
+
   let end_of_stream enc os =
     encode_buffer_float enc os [||] 0 0
 end
@@ -138,58 +151,109 @@ let opt_bs = function
   | Some bs -> bs
   | None -> -1
 
+module File = 
+struct
+  module Decoder =
+  struct
+    type t
+  
+    external create : (int -> string * int) -> (int -> Unix.seek_command -> int) -> (unit -> unit) -> (unit -> int) -> t = "ocaml_vorbis_open_dec_stream"
+  
+    let openfile_with_fd f =
+      let fd = Unix.openfile f [Unix.O_RDONLY] 0o400 in
+        try
+          create
+            (fun n ->
+               let buf = String.create n in
+               let r = Unix.read fd buf 0 n in
+                 buf, r)
+            (fun n cmd -> Unix.lseek fd n cmd)
+            (fun () -> Unix.close fd)
+            (fun () -> Unix.lseek fd 0 Unix.SEEK_CUR), fd
+        with
+          | e ->
+              Unix.close fd;
+              raise e
+  
+    let openfile f = fst (openfile_with_fd f)
+  
+    external decode_float : t -> float array array -> int -> int -> int = "ocaml_vorbis_decode_float"
+  
+    external decode_float_alloc : t -> int -> float array array = "ocaml_vorbis_decode_float_alloc"
+  
+    external decode : t -> bool -> int -> bool -> string -> int -> int -> int = "ocaml_vorbis_decode_byte" "ocaml_vorbis_decode"
+  
+    let decode df ?(big_endian=false) ?(sample_size=2) ?(signed=true) buf ofs len =
+      decode df big_endian sample_size signed buf ofs len
+  
+    external close : t -> unit = "ocaml_vorbis_close_dec_file"
+  
+    external bitstream : t -> int = "ocaml_vorbis_get_dec_file_bitstream"
+  
+    external comments : t -> int -> string * (string array) = "ocaml_vorbis_get_dec_file_comments"
+  
+    let comments df bitstream =
+      let vd, cmts = comments df bitstream in
+        vd, (Array.to_list (Array.map split_comment cmts))
+  
+    external info : t -> int -> info = "ocaml_vorbis_decoder_info"
+  
+    external bitrate : t -> int -> int = "ocaml_vorbis_decoder_bitrate"
+  
+    external duration : t -> int -> float = "ocaml_vorbis_decoder_time_total"
+  
+    external streams : t -> int = "ocaml_vorbis_decoder_streams"
+  
+    external serialnumber : t -> int -> int = "ocaml_vorbis_decoder_serialnumber"
+  
+    external samples : t -> int -> int = "ocaml_vorbis_decoder_pcm_total"
+  end
+end
+
 module Decoder =
 struct
+
   type t
 
-  external create : (int -> string * int) -> (int -> Unix.seek_command -> int) -> (unit -> unit) -> (unit -> int) -> t = "ocaml_vorbis_open_dec_stream"
+  external init : Ogg.Stream.packet -> Ogg.Stream.packet -> Ogg.Stream.packet -> t = "ocaml_vorbis_synthesis_init"
 
-  let openfile_with_fd f =
-    let fd = Unix.openfile f [Unix.O_RDONLY] 0o400 in
-      try
-        create
-          (fun n ->
-             let buf = String.create n in
-             let r = Unix.read fd buf 0 n in
-               buf, r)
-          (fun n cmd -> Unix.lseek fd n cmd)
-          (fun () -> Unix.close fd)
-          (fun () -> Unix.lseek fd 0 Unix.SEEK_CUR), fd
-      with
-        | e ->
-            Unix.close fd;
-            raise e
+  external info : t -> info = "ocaml_vorbis_val_info_of_decoder"
 
-  let openfile f = fst (openfile_with_fd f)
+  external comments : t -> string * (string array) = "ocaml_vorbis_val_comments_of_decoder" 
 
-  external decode_float : t -> float array array -> int -> int -> int = "ocaml_vorbis_decode_float"
+  let comments dec =
+    let vend,cmts = comments dec in
+      vend,Array.to_list (Array.map split_comment cmts)
 
-  external decode_float_alloc : t -> int -> float array array = "ocaml_vorbis_decode_float_alloc"
+  external check_packet : Ogg.Stream.packet -> bool = "ocaml_vorbis_check_packet"
 
-  external decode : t -> bool -> int -> bool -> string -> int -> int -> int = "ocaml_vorbis_decode_byte" "ocaml_vorbis_decode"
+  external decode_pcm : t -> Ogg.Stream.t -> float array array -> int -> int -> int = "ocaml_vorbis_decode_pcm"
 
-  let decode df ?(big_endian=false) ?(sample_size=2) ?(signed=true) buf ofs len =
-    decode df big_endian sample_size signed buf ofs len
+  let decode_pcm dec os buf pos len = 
+    let ret = decode_pcm dec os buf pos len in
+    if ret = 0 then
+      raise Ogg.Not_enough_data
+    else
+      ret
+
+  external restart : t -> unit = "ocaml_vorbis_synthesis_restart"
+  
+end
+
+module Skeleton = 
+struct
+
+  external fisbone : Nativeint.t -> Int64.t -> Int64.t -> string -> Ogg.Stream.packet = "ocaml_vorbis_skeleton_fisbone"
+
+  let fisbone ?(start_granule=Int64.zero) 
+              ?(headers=["Content-type","audio/vorbis"]) 
+              ~serialno ~samplerate () = 
+    let concat s (h,v) =
+      Printf.sprintf "%s%s: %s\r\n" s h v 
+    in
+    let s = 
+      List.fold_left concat "" headers
+    in
+    fisbone serialno samplerate start_granule s
 
-  external close : t -> unit = "ocaml_vorbis_close_dec_file"
-
-  external bitstream : t -> int = "ocaml_vorbis_get_dec_file_bitstream"
-
-  external comments : t -> int -> string * (string array) = "ocaml_vorbis_get_dec_file_comments"
-
-  let comments df bitstream =
-    let vd, cmts = comments df bitstream in
-      vd, (Array.to_list (Array.map split_comment cmts))
-
-  external info : t -> int -> info = "ocaml_vorbis_decoder_info"
-
-  external bitrate : t -> int -> int = "ocaml_vorbis_decoder_bitrate"
-
-  external duration : t -> int -> float = "ocaml_vorbis_decoder_time_total"
-
-  external streams : t -> int = "ocaml_vorbis_decoder_streams"
-
-  external serialnumber : t -> int -> int = "ocaml_vorbis_decoder_serialnumber"
-
-  external samples : t -> int -> int = "ocaml_vorbis_decoder_pcm_total"
 end
